import click

from ..repo import RepoManager


def repo_manager(ctx) -> RepoManager:
    return RepoManager(ctx.obj["agile"])


def get_repos(labels):
    if not labels:
        click.echo(
            'You need to specify the "labels" object in the config file', error=True
        )
        click.exit()
    repos = labels.get("repositories")
    if not isinstance(repos, list):
        click.echo(
            'You need to specify the "repos" list in the config ' "labels object file",
            error=True,
        )
        click.exit()
    return repos



import logging
import os
from typing import Dict

import requests

from .components import GithubException
from .repo import GitRepo

__all__ = ["GithubApi", "GitRepo", "GithubException"]


LOGGER = logging.getLogger("github")


class GithubApi:
    def __init__(self, token: str = "") -> None:
        self.http = requests.Session()
        self.token = token or get_token()

    @property
    def api_url(self):
        return "https://api.github.com"

    @property
    def uploads_url(self):
        return "https://uploads.github.com"

    def __repr__(self):
        return self.api_url

    __str__ = __repr__

    def repo(self, repo_path):
        return GitRepo(self, repo_path)

    def default_headers(self) -> Dict[str, str]:
        headers = {}
        if self.token:
            headers["authorization"] = f"token {self.token}"
        return headers


def get_token() -> str:
    token = os.getenv("GITHUB_SECRET_TOKEN") or os.getenv("GITHUB_TOKEN")
    return token



from unittest import mock

from agiletoolkit.repo import RepoManager
from agiletoolkit.test import gitrepo


def test_message():
    with gitrepo("slack"):
        m = RepoManager()
        with mock.patch("click.secho") as p:
            m.message("test")
            assert p.called



from typing import Dict

from ..utils import CommandError


class GithubException(CommandError):
    pass


class Component:
    """Base class for github components
    """

    def __init__(self, client):
        self.client = client

    def __repr__(self):
        return self.api_url

    __str__ = __repr__

    def __getattr__(self, name):
        return getattr(self.client, name)

    @classmethod
    def id_from_data(cls, data):
        return data["id"]

    @classmethod
    def as_id(cls, data):
        if isinstance(data, dict):
            return cls.id_from_data(data)
        return data

    def default_headers(self) -> Dict[str, str]:
        return self.client.default_headers()


class RepoComponents(Component):
    @property
    def api_url(self):
        return "%s/%s" % (self.client, self.__class__.__name__.lower())

    def get(self, id):
        """Get data for this component
        """
        id = self.as_id(id)
        url = "%s/%s" % (self, id)
        response = self.http.get(url, headers=self.default_headers())
        response.raise_for_status()
        return response.json()

    def create(self, data):
        """Create a new component
        """
        response = self.http.post(str(self), json=data, headers=self.default_headers())
        response.raise_for_status()
        return response.json()

    def update(self, id, data):
        """Update a component
        """
        id = self.as_id(id)
        response = self.http.patch(
            "%s/%s" % (self, id), json=data, headers=self.default_headers()
        )
        response.raise_for_status()
        return response.json()

    def delete(self, id):
        """Delete a component by id
        """
        id = self.as_id(id)
        response = self.http.delete(
            "%s/%s" % (self.api_url, id), headers=self.default_headers()
        )
        response.raise_for_status()

    def get_list(self, url=None, callback=None, limit=100, **data):
        """Get a list of this github component
        :param url: full url
        :param Comp: a :class:`.Component` class
        :param callback: Optional callback
        :param limit: Optional number of items to retrieve
        :param data: additional query data
        :return: a list of ``Comp`` objects with data
        """
        url = url or str(self)
        data = dict(((k, v) for k, v in data.items() if v))
        all_data = []
        if limit:
            data["per_page"] = min(limit, 100)
        while url:
            response = self.http.get(url, params=data, headers=self.default_headers())
            response.raise_for_status()
            result = response.json()
            n = m = len(result)
            if callback:
                result = callback(result)
                m = len(result)
            all_data.extend(result)
            if limit and len(all_data) > limit:
                all_data = all_data[:limit]
                break
            elif m == n:
                data = None
                next = response.links.get("next", {})
                url = next.get("url")
            else:
                break
        return all_data


class RepoComponentsId(RepoComponents):
    def __init__(self, root, id):
        super().__init__(root)
        self.id = id

    @property
    def api_url(self):
        return "%s/%s/%s" % (self.client, self.id, self.__class__.__name__.lower())


class Commits(RepoComponents):
    @classmethod
    def id_from_data(cls, data):
        return data["sha"]

    def comments(self, commit):
        """Fetch comments for a given commit
        """
        commit = self.as_id(commit)
        return self.get_list(url="%s/%s/comments" % (self, commit))


class Issues(RepoComponents):
    @classmethod
    def id_from_data(cls, data):
        return data["number"]

    def comments(self, issue):
        """Return all comments for this issue/pull request
        """
        commit = self.as_id(issue)
        return self.get_list(url="%s/%s/comments" % (self, commit))


class Comments(RepoComponentsId):
    pass


class Pulls(Issues):
    pass


class Milestones(Issues):
    pass


class Hooks(RepoComponents):
    pass



import click

from ..api import GithubApi
from ..utils import CommandError


@click.command()
@click.pass_context
def labels(ctx):
    """Crate or update labels in github
    """
    config = ctx.obj["agile"]
    repos = config.get("repositories")
    labels = config.get("labels")
    if not isinstance(repos, list):
        raise CommandError('You need to specify the "repos" list in the config')
    if not isinstance(labels, dict):
        raise CommandError('You need to specify the "labels" dictionary in the config')
    git = GithubApi()
    for repo in repos:
        repo = git.repo(repo)
        for label, color in labels.items():
            if repo.label(label, color):
                click.echo('Created label "%s" @ %s' % (label, repo))
            else:
                click.echo('Updated label "%s" @ %s' % (label, repo))



import json
import os

import click

from . import __version__
from .github import git
from .kong import kong
from .utils import niceJson

AGILE_CONFIG = os.environ.get("AGILE_CONFIG", "agile.json")


@click.group(invoke_without_command=True)
@click.option(
    "--debug/--no-debug", is_flag=True, default=False, help="Run in debug mode"
)
@click.option("--version", is_flag=True, default=False, help="Display version and exit")
@click.option(
    "--show-config", is_flag=True, default=False, help="Display config and exit"
)
@click.option(
    "--config",
    default=AGILE_CONFIG,
    type=click.Path(),
    help=f"Agile configuration json file location ({AGILE_CONFIG})",
)
@click.pass_context
def start(
    ctx: click.Context, debug: bool, version: bool, show_config: bool, config: str
):
    """Commands for devops operations"""
    ctx.obj = {}
    ctx.DEBUG = debug
    if os.path.isfile(config):
        with open(config) as fp:
            agile = json.load(fp)
    else:
        agile = {}
    ctx.obj["agile"] = agile
    if version or show_config:
        if version:
            click.echo(__version__)
        if show_config:
            click.echo(niceJson(agile))
        ctx.exit(0)
    if not ctx.invoked_subcommand:
        click.echo(ctx.get_help())


start.add_command(git)
start.add_command(kong)



import click

from kong.client import Kong, KongError

from . import utils
from .repo import RepoManager


@click.command()
@click.pass_context
@click.option("--namespace", default="dev", help="target namespace")
@click.option("--yes", is_flag=True, help="commit changes to kong", default=False)
def kong(ctx, namespace, yes):
    """Update the kong configuration
    """
    m = KongManager(ctx.obj["agile"], namespace=namespace)
    click.echo(utils.niceJson(m.create_kong(yes)))


class KongManager(RepoManager):
    def create_kong(self, yes):
        data = self.load_data("values.yaml")
        values = data.copy()
        manifest = self.manifest(values, "kong.yaml")
        if yes:
            return self.wait(self.apply_kong(manifest))
        else:
            return manifest

    async def apply_kong(self, manifest):
        async with Kong() as cli:
            try:
                result = await cli.apply_json(manifest)
            except KongError as exc:
                raise utils.CommandError(f"Kong Error: {exc}") from None
            return result



import click

from .info import auth, info
from .labels import labels
from .milestones import milestones
from .release import release
from .remote import remote
from .validate import validate


@click.group(invoke_without_command=True)
@click.pass_context
def git(ctx):
    """Github client command repo management
    """
    if not ctx.invoked_subcommand:
        click.echo(ctx.get_help())


git.add_command(info)
git.add_command(auth)
git.add_command(labels)
git.add_command(milestones)
git.add_command(release)
git.add_command(remote)
git.add_command(validate)



import click

from ..api import GithubApi
from .utils import get_repos


@click.command()
@click.pass_context
@click.option("--list", is_flag=True, help="list open milestones", default=False)
@click.option("--close", help="milestone to close", default="")
def milestones(ctx, list, close):
    """View/edit/close milestones on github
    """
    repos = get_repos(ctx.parent.agile.get("labels"))
    if list:
        _list_milestones(repos)
    elif close:
        click.echo('Closing milestones "%s"' % close)
        _close_milestone(repos, close)
    else:
        click.echo(ctx.get_help())


def _list_milestones(repos):
    git = GithubApi()
    milestones = set()
    for repo in repos:
        repo = git.repo(repo)
        stones = repo.milestones.get_list()
        milestones.update((data["title"] for data in stones))
    for title in sorted(milestones):
        click.echo(title)


def _close_milestone(repos, milestone):
    git = GithubApi()
    for repo in repos:
        repo = git.repo(repo)
        _close_repo_milestone(repo, milestone)


def _close_repo_milestone(repo, milestone):
    milestones = repo.milestones.get_list()
    for m in milestones:
        if m["title"] == milestone:
            repo.milestones.update(m, {"state": "closed"})
            click.echo("Closed milestone %s" % m["html_url"])



from typing import Dict

from .components import Commits, Component, Hooks, Issues, Milestones, Pulls
from .releases import Releases


class GitRepo(Component):
    """Github repository endpoints
    """

    def __init__(self, client, repo_path):
        super().__init__(client)
        self.repo_path = repo_path
        self.releases = Releases(self)
        self.commits = Commits(self)
        self.issues = Issues(self)
        self.pulls = Pulls(self)
        self.milestones = Milestones(self)
        self.hooks = Hooks(self)

    @property
    def api_url(self):
        return "%s/repos/%s" % (self.client, self.repo_path)

    def default_headers(self) -> Dict[str, str]:
        headers = super().default_headers()
        headers.update(accept="application/vnd.github.symmetra-preview+json")
        return headers

    def label(self, name, color, update=True):
        """Create or update a label
        """
        url = "%s/labels" % self
        data = dict(name=name, color=color)
        response = self.http.post(url, json=data, headers=self.default_headers())
        if response.status_code == 201:
            return True
        elif response.status_code == 422 and update:
            url = "%s/%s" % (url, name)
            response = self.http.patch(url, json=data, headers=self.default_headers())
        response.raise_for_status()
        return False



import os

from slack import WebClient


class SlackIntegration:
    token = os.environ.get("SLACK_TOKEN")

    def __init__(self, manager):
        self.manager = manager
        self.channel = manager.config.get("slack_channel", "agile")
        self.cli = WebClient(token=self.token)

    @classmethod
    def add(cls, manager):
        if cls.token:
            manager.message_brokers.append(cls(manager))

    def __call__(self, message):
        info = self.manager.info
        repo = self.manager.github_repo().repo_path
        sha = info["head"]["id"]
        commit = "https://github.com/%s/commit/%s" % (repo, sha)
        message = "*%s* - %s\nby %s <%s>\n%s" % (
            repo,
            info["branch"],
            info["head"]["committer_name"],
            info["head"]["committer_email"],
            message,
        )
        attachments = [
            dict(
                fallback="Git commit %s" % commit,
                actions=[dict(type="button", text="Git commit", url=commit)],
            )
        ]

        self.cli.api_call(
            "chat.postMessage",
            channel=self.channel,
            text=message,
            attachments=attachments,
        )



import json

import click

from ..utils import gitrepo
from .utils import repo_manager


@click.command()
def info():
    """Display information about repository
    """
    info = gitrepo()
    click.echo(json.dumps(info, indent=4))


@click.command()
@click.pass_context
def auth(ctx):
    """Display github token
    """
    repo = repo_manager(ctx)
    click.echo(repo.github.token)



import yaml
from click.testing import CliRunner

from agiletoolkit import __version__
from agiletoolkit.api import GithubException
from agiletoolkit.commands import start
from agiletoolkit.repo import RepoManager
from agiletoolkit.test import gitrepo


def mocked(*a, **kw):
    print("mocked")


def test_main():
    runner = CliRunner()
    result = runner.invoke(start)
    assert result.exit_code == 0
    assert result.output.startswith("Usage:")
    #
    result = runner.invoke(start, ["--config", "tests/cfg1.json"])
    assert result.exit_code == 0
    assert result.output.startswith("Usage:")


def test_version():
    runner = CliRunner()
    result = runner.invoke(start, ["--version"])
    assert result.exit_code == 0
    assert result.output.strip() == __version__


def test_repo(mocker):
    with gitrepo("master"):
        m = RepoManager()
        m.validate_version = mocker.Mock(return_value=True)
        assert m.can_release()

    with gitrepo("master"):
        m = RepoManager()
        m.validate_version = mocker.Mock(side_effect=GithubException("test"))
        assert m.can_release() is False


def test_docker_tag():
    with gitrepo("feature/ABC-123/bla-bla", head_id="1234567890"):
        m = RepoManager()
        assert m.version() == "feature-abc-123-bla-bla-12345678"


def test_load_data(mocker):
    with gitrepo("deploy"):
        m = RepoManager(namespace="dev")
        with open("./deploy/values.yaml", "w") as f:
            f.write(
                yaml.dump(
                    {
                        "foo": "1",
                        "bar": "2",
                        "dev": {"bar": "3"},
                        "production": {"bar": "4"},
                    }
                )
            )
        data = m.load_data("deploy", "values.yaml")
        expected = {
            "foo": "1",
            "bar": "3",
            "namespace": "dev",
        }
        assert data == expected



import copy
import os
import tempfile
from contextlib import contextmanager
from unittest import mock

from .api import GithubException

GIT_OUTPUT = {
    "branch": "add_namespace",
    "current_tag": "",
    "head": {
        "author_email": "konrad@example.com",
        "author_name": "Konrad Rotkiewicz",
        "committer_email": "konrad@example.com",
        "committer_name": "Konrad Rotkiewicz",
        "id": "22f17e5c69dcda3b6695ac07b9f32e3c4964848a",
        "message": "add namespace when loading data",
    },
    "pr": False,
    "remotes": [
        {"name": "origin", "url": "git@github.com:quantmind/agile-toolkit.git"},
    ],
    "tag": "v0.1.3",
}


MAKEFILE = """\
version:
\t@echo "1.2.3"
"""


CODEBUILD = """\
name: test
description: test
source:
  location: https://github.com/example/example.git
"""


@contextmanager
def gitrepo(branch: str, pr=False, tag=None, head_id=None):
    """
    prepare artificial git repo
    1. create temp dir
    2. add basic project structure
    3. cd to that dir
    4. mock utils.gtirepo
    5. cd back and removes the dir
    """

    def mocker(root=None):
        data = copy.deepcopy(GIT_OUTPUT)
        data["branch"] = branch
        data["pr"] = pr
        data["tag"] = tag
        if head_id:
            data["head"]["id"] = head_id
        return data

    with tempfile.TemporaryDirectory() as temp_dir:
        curr_dir = os.getcwd()
        os.chdir(temp_dir)
        os.makedirs("deploy")
        with open("Makefile", "w") as f:
            f.write(MAKEFILE)
        with mock.patch("agiletoolkit.utils.gitrepo", side_effect=mocker) as m:
            try:
                yield m
            finally:
                os.chdir(curr_dir)


def github_error(*args, **kwargs):
    raise GithubException



import logging
import mimetypes
import os
import stat
from urllib.parse import urlsplit

from ..utils import semantic_version
from .components import GithubException, RepoComponents

logger = logging.getLogger(__file__)

ONEMB = 2 ** 20


class Assets(RepoComponents):
    pass


class Releases(RepoComponents):
    """Github repository endpoints
    """

    @property
    def assets(self):
        return Assets(self)

    def latest(self):
        """Get the latest release of this repo
        """
        url = "%s/latest" % self
        response = self.http.get(url, headers=self.default_headers())
        if response.status_code == 200:
            return response.json()
        elif response.status_code != 404:
            response.raise_for_status()

    def tag(self, tag):
        """Get a release by tag
        """
        url = "%s/tags/%s" % (self, tag)
        response = self.http.get(url, headers=self.default_headers())
        response.raise_for_status()
        return response.json()

    def delete(self, id=None, tag=None):
        if tag:
            assert not id, "provide either tag or id to delete"
            release = self.tag(tag)
            id = release["id"]
        assert id, "id not given"
        return super().delete(id)

    def release_assets(self, release):
        """Assets for a given release
        """
        release = self.as_id(release)
        return self.get_list(url="%s/%s/assets" % (self, release))

    def upload(self, release, filename, content_type=None):
        """Upload a file to a release

        :param filename: filename to upload
        :param content_type: optional content type
        :return: json object from github
        """
        release = self.as_id(release)
        name = os.path.basename(filename)
        if not content_type:
            content_type, _ = mimetypes.guess_type(name)
        if not content_type:
            raise ValueError("content_type not known")
        inputs = {"name": name}
        url = "%s%s/%s/assets" % (
            self.uploads_url,
            urlsplit(self.api_url).path,
            release,
        )
        info = os.stat(filename)
        size = info[stat.ST_SIZE]
        headers = (self.default_headers(),)
        headers.update({"content-type": content_type, "content-length": str(size)})
        response = self.http.post(
            url, data=stream_upload(filename), params=inputs, headers=headers,
        )
        response.raise_for_status()
        return response.json()

    def validate_tag(self, tag_name, prefix=None):
        """Validate ``tag_name`` with the latest tag from github

        If ``tag_name`` is a valid candidate, return the latest tag from github
        """
        new_version = semantic_version(tag_name)
        current = self.latest()
        if current:
            tag_name = current["tag_name"]
            if prefix:
                tag_name = tag_name[len(prefix) :]
            tag_name = semantic_version(tag_name)
            if tag_name >= new_version:
                what = "equal to" if tag_name == new_version else "older than"
                raise GithubException(
                    'Your local version "%s" is %s '
                    'the current github version "%s".\n'
                    "Bump the local version to "
                    "continue." % (str(new_version), what, str(tag_name))
                )
        return current


def stream_upload(filename):
    with open(filename, "rb") as file:
        while True:
            chunk = file.read(ONEMB)
            if not chunk:
                break
            yield chunk



import click

from ..api import GithubException
from .utils import repo_manager


@click.command()
@click.option(
    "--sandbox",
    is_flag=True,
    help="Validate only on sandbox/deploy branch",
    default=False,
)
@click.option(
    "--yes-no",
    is_flag=True,
    help="Return yes for version is good, no for version is not good",
    default=False,
)
@click.pass_context
def validate(ctx, sandbox, yes_no):
    """Check if version of repository is semantic
    """
    m = repo_manager(ctx)
    if not sandbox or m.can_release("sandbox"):
        try:
            version = m.validate_version()
            click.echo("yes" if yes_no else version)
        except GithubException:
            if yes_no:
                return click.echo("no")
            raise



import os
import sys

from setuptools import find_packages, setup

import agiletoolkit as pkg


def read(name):
    filename = os.path.join(os.path.dirname(__file__), name)
    with open(filename) as fp:
        return fp.read()


def requirements(name):
    install_requires = []
    dependency_links = []

    for line in read(name).split("\n"):
        if line.startswith("-e "):
            link = line[3:].strip()
            if link == ".":
                continue
            dependency_links.append(link)
            line = link.split("=")[1]
        line = line.strip()
        if line:
            install_requires.append(line)

    return install_requires, dependency_links


install_requires = requirements("dev/requirements.txt")[0]


if sys.version_info < (3, 7):
    install_requires.append("dataclasses")


meta = dict(
    version=pkg.__version__,
    description=pkg.__doc__,
    name="agile-toolkit",
    author="Luca Sbardella",
    author_email="luca@quantmind.com",
    maintainer_email="luca@quantmind.com",
    url="https://github.com/quantmind/agile-toolkit",
    long_description=read("readme.md"),
    long_description_content_type="text/markdown",
    packages=find_packages(exclude=["tests", "tests.*"]),
    include_package_data=True,
    zip_safe=False,
    install_requires=install_requires,
    entry_points={"console_scripts": ["agilekit=agiletoolkit.commands:start"]},
    classifiers=[
        "Development Status :: 3 - Alpha",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: BSD License",
        "Operating System :: OS Independent",
        "Programming Language :: JavaScript",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Topic :: Utilities",
    ],
)


if __name__ == "__main__":
    setup(**meta)



import click

from ..utils import niceJson
from .utils import repo_manager


@click.command()
@click.option("--yes", is_flag=True, help="Commit changes to github", default=False)
@click.option(
    "--latest", is_flag=True, help="Show latest release in github", default=False
)
@click.pass_context
def release(ctx, yes, latest):
    """Create a new release in github
    """
    m = repo_manager(ctx)
    api = m.github_repo()
    if latest:
        latest = api.releases.latest()
        if latest:
            click.echo(latest["tag_name"])
        else:
            click.echo("no releases")
    elif m.can_release("sandbox"):
        branch = m.info["branch"]
        version = m.validate_version()
        name = "v%s" % version
        body = ["Release %s from agiletoolkit" % name]
        data = dict(
            tag_name=name,
            target_commitish=branch,
            name=name,
            body="\n\n".join(body),
            draft=False,
            prerelease=False,
        )
        if yes:
            data = api.releases.create(data=data)
            m.message("Successfully created a new Github release")
        click.echo(niceJson(data))
    else:
        click.echo("skipped")



import os
import re
from dataclasses import dataclass
from urllib.parse import urlparse

from . import utils
from .api import GithubApi, GithubException
from .manager import Manager
from .slack import SlackIntegration


@dataclass
class Branches:
    dev: str = "master"
    sandbox: str = "master"


class RepoManager(Manager):
    def init(self):
        self.info = utils.gitrepo(self.path)
        SlackIntegration.add(self)

    @property
    def github(self):
        return GithubApi()

    def software_version(self):
        """Software version

        Requires the Makefile entry

        version:
            ....
        """
        return utils.version()

    def version(self):
        """Software version of the current repository
        """
        branches = self.branches()
        if self.info["branch"] == branches.sandbox:
            try:
                return self.software_version()
            except Exception as exc:
                raise utils.CommandError(
                    "Could not obtain repo version, do you have a makefile "
                    "with version entry?\n%s" % exc
                )
        else:
            branch = self.info["branch"].lower()
            branch = re.sub("[^a-z0-9_-]+", "-", branch)
            return f"{branch}-{self.info['head']['id'][:8]}"

    def validate_version(self, prefix: str = "v", yes_no: bool = False):
        """Validate version by checking if it is a valid semantic version
        and its value is higher than latest github tag
        """
        version = self.software_version()
        repo = self.github_repo()
        repo.releases.validate_tag(version, prefix)
        return version

    def skip_build(self):
        """Check if build should be skipped
        """
        skip_msg = self.config.get("skip", "[ci skip]")
        return (
            os.environ.get("CODEBUILD_BUILD_SUCCEEDING") == "0"
            or self.info["current_tag"]
            or skip_msg in self.info["head"]["message"]
        )

    def can_release(self, target=None):
        if self.skip_build() or self.info["pr"]:
            return False

        branch = self.info["branch"]
        branches = self.branches()

        if branch not in (branches.dev, branches.sandbox):
            return False

        if target and branch != getattr(branches, target):
            return False

        if not target:
            target = "sandbox" if branch == branches.sandbox else "dev"

        if target == "sandbox":
            try:
                self.validate_version()
            except GithubException:
                return False

        return True

    def branches(self):
        return self.get("branches", Branches)

    def target_from_branch(self):
        branches = self.branches()
        return "dev" if self.info["branch"] == branches.dev else "sandbox"

    def github_repo(self) -> str:
        url = self.info["remotes"][0]["url"]
        if url.startswith("git@"):
            url = url.split(":")
            assert url[0] == "git@github.com"
            path = url[1]
        else:
            path = urlparse(url).path[1:]
        if path.endswith(".git"):
            path = path[:-4]
        return self.github.repo(path)



import json
from unittest import mock

from click.testing import CliRunner

from agiletoolkit.commands import start
from agiletoolkit.test import gitrepo

from .utils import mock_response


def test_git():
    runner = CliRunner()
    result = runner.invoke(start, ["git"])
    assert result.exit_code == 0
    assert result.output.startswith("Usage:")


def test_git_validate():
    runner = CliRunner()
    with gitrepo("deploy"):
        result = runner.invoke(start, ["git", "validate"])
        print(result.output)
        assert result.exit_code == 0
        assert result.output.strip() == "1.2.3"


def test_git_info():
    runner = CliRunner()
    result = runner.invoke(start, ["git", "info"])
    assert result.exit_code == 0
    data = json.loads(result.output.strip())
    assert "branch" in data


def test_git_remote():
    runner = CliRunner()
    result = runner.invoke(start, ["git", "remote"])
    assert result.exit_code == 0
    assert result.output.strip() == "quantmind/agile-toolkit"


def test_git_release_latest():
    runner = CliRunner()
    result = runner.invoke(start, ["git", "release", "--latest"])
    assert result.exit_code == 0
    assert result.output.strip().startswith("v")


def test_git_release_skipped():
    runner = CliRunner()

    with gitrepo("dev") as mock:
        result = runner.invoke(start, ["git", "release"])
        assert result.exit_code == 0
        assert mock.called
        assert result.output.strip() == "skipped"


def test_labels_error():
    runner = CliRunner()
    result = runner.invoke(start, ["--config", "tests/cfg1.json", "git", "labels"])
    assert result.exit_code == 1
    assert result.output == (
        'Error: You need to specify the "repos" list in the config\n'
    )
    result = runner.invoke(start, ["--config", "tests/cfg2.json", "git", "labels"])
    assert result.exit_code == 1
    assert result.output == (
        'Error: You need to specify the "labels" dictionary in the config\n'
    )


def test_git_labels():
    runner = CliRunner()
    with mock.patch("requests.Session") as ses:
        ses.return_value = mock_response()
        result = runner.invoke(start, ["--config", "tests/cfg3.json", "git", "labels"])
        assert result.exit_code == 0
